@@ -2,15 +2,122 @@
 A collection of utilities for testing webgrid functionality in client applications
 """
 import re
+from unittest import mock
 import urllib
 import warnings
 
 import flask
 import openpyxl
 from pyquery import PyQuery
-import xlrd
+import sqlalchemy
+try:
+    import xlrd
+except ImportError:
+    pass
 
-from webgrid.tests import helpers
+
+def compiler_instance_factory(compiler, dialect, statement):  # noqa: C901
+    class LiteralCompiler(compiler.__class__):
+        def render_literal_value(self, value, type_):
+            import datetime
+            """
+            For date and datetime values, convert to a string
+            format acceptable to the dialect. That seems to be the
+            so-called ODBC canonical date format which looks
+            like this:
+
+                yyyy-mm-dd hh:mi:ss.mmm(24h)
+
+            For other data types, call the base class implementation.
+            """
+            if isinstance(value, datetime.datetime):
+                return "'" + value.strftime('%Y-%m-%d %H:%M:%S.%f') + "'"
+            elif isinstance(value, datetime.date):
+                return "'" + value.strftime('%Y-%m-%d') + "'"
+            elif isinstance(value, datetime.time):
+                return "'{:%H:%M:%S.%f}'".format(value)
+            elif value is None:
+                return 'NULL'
+            else:
+                # Turn off double percent escaping, since we don't run these strings and
+                # it creates a large number of differences for test cases
+                with mock.patch.object(
+                    dialect.identifier_preparer,
+                    '_double_percents',
+                    False
+                ):
+                    return super(LiteralCompiler, self).render_literal_value(value, type_)
+
+        def visit_bindparam(
+                self, bindparam, within_columns_clause=False,
+                literal_binds=False, **kwargs
+        ):
+            return super(LiteralCompiler, self).render_literal_bindparam(
+                bindparam, within_columns_clause=within_columns_clause,
+                literal_binds=literal_binds, **kwargs
+            )
+
+        def visit_table(self, table, asfrom=False, iscrud=False, ashint=False,
+                        fromhints=None, use_schema=True, **kwargs):
+            """Strip the default schema from table names when it is not needed"""
+            ret_val = super().visit_table(table, asfrom, iscrud, ashint, fromhints, use_schema,
+                                          **kwargs)
+            if dialect.name == 'mssql' and ret_val.startswith('dbo.'):
+                return ret_val[4:]
+            return ret_val
+
+        def visit_column(self, column, add_to_result_map=None, include_table=True, **kwargs):
+            """Strip the default schema from table names when it is not needed"""
+            ret_val = super().visit_column(column, add_to_result_map, include_table, **kwargs)
+            if dialect.name == 'mssql' and ret_val.startswith('dbo.'):
+                return ret_val[4:]
+            return ret_val
+
+    return LiteralCompiler(dialect, statement)
+
+
+def query_to_str(statement, bind=None):
+    """
+        returns a string of a sqlalchemy.orm.Query with parameters bound
+
+        WARNING: this is dangerous and ONLY for testing, executing the results
+        of this function can result in an SQL Injection attack.
+    """
+    if isinstance(statement, sqlalchemy.orm.Query):
+        if bind is None:
+            bind = statement.session.get_bind(
+                statement._mapper_zero()
+            )
+        statement = statement.statement
+    elif bind is None:
+        bind = statement.bind
+
+    if bind is None:
+        raise Exception('bind param (engine or connection object) required when using with an'
+                        ' unbound statement')
+
+    dialect = bind.dialect
+    compiler = statement._compiler(dialect)
+    literal_compiler = compiler_instance_factory(compiler, dialect, statement)
+    return 'TESTING ONLY BIND: ' + literal_compiler.process(statement)
+
+
+def assert_in_query(obj, test_for):
+    if hasattr(obj, 'build_query'):
+        query = obj.build_query()
+    else:
+        query = obj
+    query_str = query_to_str(query)
+    assert test_for in query_str, query_str
+
+
+def assert_not_in_query(obj, test_for):
+    if hasattr(obj, 'build_query'):
+        query = obj.build_query()
+    else:
+        query = obj
+    query_str = query_to_str(query)
+    assert test_for not in query_str, query_str
 
 
 def assert_list_equal(list1, list2):
@@ -178,7 +285,7 @@ class GridBase:
 
     def query_to_str(self, statement, bind=None):
         """Render a SQLAlchemy query to a string."""
-        return helpers.query_to_str(statement, bind=bind)
+        return query_to_str(statement, bind=bind)
 
     def assert_in_query(self, look_for, grid=None, **kwargs):
         """Verify the given SQL string is in the grid's query.
@@ -192,7 +299,7 @@ class GridBase:
             kwargs (dict, optional): Additional args passed to `self.get_session_grid`.
         """
         grid = grid or self.get_session_grid(**kwargs)
-        helpers.assert_in_query(grid, look_for)
+        assert_in_query(grid, look_for)
 
     def assert_not_in_query(self, look_for, grid=None, **kwargs):
         """Verify the given SQL string is not in the grid's query.
@@ -206,7 +313,7 @@ class GridBase:
             kwargs (dict, optional): Additional args passed to `self.get_session_grid`.
         """
         grid = grid or self.get_session_grid(**kwargs)
-        helpers.assert_not_in_query(grid, look_for)
+        assert_not_in_query(grid, look_for)
 
     def assert_regex_in_query(self, look_for, grid=None, **kwargs):
         """Verify the given regex matches the grid's query.
